require "endorser/receipt/version"

module Endorser
  class Receipt
    attr_reader :request_method
    attr_reader :headers

    attr_accessor :api_key
    attr_accessor :api_secret
    attr_accessor :body
    attr_accessor :content_type
    attr_accessor :uri
    attr_accessor :host

    def initialize(params={})
      @headers = {}

      self.api_key      = params.fetch(:api_key, nil)
      self.api_secret   = params.fetch(:api_secret, nil)
      self.body         = params.fetch(:body, nil)
      self.content_type = params.fetch(:content_type, nil)
      self.uri          = params.fetch(:uri, nil)
      self.host         = params.fetch(:host, nil)
      request_method    = params.fetch(:request_method, '')
    end

    def request_method=(s)
      @request_method = s.upcase
    end

    def body_md5
      Digest::MD5.hexdigest(body)
    end

    def body_length
      body.length
    end

    def to_s
      preamble + header_text
    end

    def preamble
      text = ""
      text << "%s\n" % request_method
      text << "%s\n" % host
      text << "%s\n" % uri
      text << "%s\n" % api_key
      text << "%s\n" % content_type
      text << "%s\n" % body_length
      text << "%s\n" % body_md5
      text
    end

    def header_text
      text = ""
      headers.sort_by { |k,v| k.downcase }.each do |header, value|
        text << "%s:%s\n" % [header.downcase, value]
      end
      text
    end
  end
end
